@@ -45,6 +45,8 @@ GST_DEBUG_CATEGORY_STATIC (gst_audio_clock_debug);
 static void gst_audio_clock_class_init (GstAudioClockClass * klass);
 static void gst_audio_clock_init (GstAudioClock * clock);
 
+static void gst_audio_clock_dispose (GObject * object);
+
 static GstClockTime gst_audio_clock_get_internal_time (GstClock * clock);
 
 static GstSystemClockClass *parent_class = NULL;
@@ -54,42 +56,41 @@ static GstSystemClockClass *parent_class = NULL;
 GType
 gst_audio_clock_get_type (void)
 {
-  static GType clock_type = 0;
+  static volatile gsize clock_type = 0;
+  static const GTypeInfo clock_info = {
+    sizeof (GstAudioClockClass),
+    NULL,
+    NULL,
+    (GClassInitFunc) gst_audio_clock_class_init,
+    NULL,
+    NULL,
+    sizeof (GstAudioClock),
+    4,
+    (GInstanceInitFunc) gst_audio_clock_init,
+    NULL
+  };
 
-  if (!clock_type) {
-    static const GTypeInfo clock_info = {
-      sizeof (GstAudioClockClass),
-      NULL,
-      NULL,
-      (GClassInitFunc) gst_audio_clock_class_init,
-      NULL,
-      NULL,
-      sizeof (GstAudioClock),
-      4,
-      (GInstanceInitFunc) gst_audio_clock_init,
-      NULL
-    };
-
-    clock_type = g_type_register_static (GST_TYPE_SYSTEM_CLOCK, "GstAudioClock",
+  if (g_once_init_enter (&clock_type)) {
+    GType tmp = g_type_register_static (GST_TYPE_SYSTEM_CLOCK, "GstAudioClock",
         &clock_info, 0);
+    g_once_init_leave (&clock_type, tmp);
   }
-  return clock_type;
-}
 
+  return (GType) clock_type;
+}
 
 static void
 gst_audio_clock_class_init (GstAudioClockClass * klass)
 {
-  GObjectClass *gobject_class;
-  GstObjectClass *gstobject_class;
   GstClockClass *gstclock_class;
+  GObjectClass *gobject_class;
 
   gobject_class = (GObjectClass *) klass;
-  gstobject_class = (GstObjectClass *) klass;
   gstclock_class = (GstClockClass *) klass;
 
   parent_class = g_type_class_peek_parent (klass);
 
+  gobject_class->dispose = gst_audio_clock_dispose;
   gstclock_class->get_internal_time = gst_audio_clock_get_internal_time;
 
   GST_DEBUG_CATEGORY_INIT (gst_audio_clock_debug, "audioclock", 0,
@@ -99,11 +100,25 @@ gst_audio_clock_class_init (GstAudioClockClass * klass)
 static void
 gst_audio_clock_init (GstAudioClock * clock)
 {
+  GST_DEBUG_OBJECT (clock, "init");
   clock->last_time = 0;
   clock->abidata.ABI.time_offset = 0;
   GST_OBJECT_FLAG_SET (clock, GST_CLOCK_FLAG_CAN_SET_MASTER);
 }
 
+static void
+gst_audio_clock_dispose (GObject * object)
+{
+  GstAudioClock *clock = GST_AUDIO_CLOCK (object);
+
+  if (clock->abidata.ABI.destroy_notify && clock->user_data)
+    clock->abidata.ABI.destroy_notify (clock->user_data);
+  clock->abidata.ABI.destroy_notify = NULL;
+  clock->user_data = NULL;
+
+  G_OBJECT_CLASS (parent_class)->dispose (object);
+}
+
 /**
  * gst_audio_clock_new:
  * @name: the name of the clock
@@ -129,6 +144,35 @@ gst_audio_clock_new (const gchar * name, GstAudioClockGetTimeFunc func,
   return (GstClock *) aclock;
 }
 
+/**
+ * gst_audio_clock_new_full:
+ * @name: the name of the clock
+ * @func: a function
+ * @user_data: user data
+ * @destroy_notify: #GDestroyNotify for @user_data
+ *
+ * Create a new #GstAudioClock instance. Whenever the clock time should be
+ * calculated it will call @func with @user_data. When @func returns
+ * #GST_CLOCK_TIME_NONE, the clock will return the last reported time.
+ *
+ * Returns: a new #GstAudioClock casted to a #GstClock.
+ *
+ * Since: 0.10.31
+ */
+GstClock *
+gst_audio_clock_new_full (const gchar * name, GstAudioClockGetTimeFunc func,
+    gpointer user_data, GDestroyNotify destroy_notify)
+{
+  GstAudioClock *aclock =
+      GST_AUDIO_CLOCK (g_object_new (GST_TYPE_AUDIO_CLOCK, "name", name, NULL));
+
+  aclock->func = func;
+  aclock->user_data = user_data;
+  aclock->abidata.ABI.destroy_notify = destroy_notify;
+
+  return (GstClock *) aclock;
+}
+
 /**
  * gst_audio_clock_reset:
  * @clock: a #GstAudioClock
@@ -152,8 +196,15 @@ gst_audio_clock_reset (GstAudioClock * clock, GstClockTime time)
   clock->abidata.ABI.time_offset = time_offset;
 
   GST_DEBUG_OBJECT (clock,
-      "reset clock to %" GST_TIME_FORMAT ", offset %" GST_TIME_FORMAT,
-      GST_TIME_ARGS (time), GST_TIME_ARGS (time_offset));
+      "reset clock to %" GST_TIME_FORMAT ", last %" GST_TIME_FORMAT ", offset %"
+      GST_TIME_FORMAT, GST_TIME_ARGS (time), GST_TIME_ARGS (clock->last_time),
+      GST_TIME_ARGS (time_offset));
+}
+
+static GstClockTime
+gst_audio_clock_func_invalid (GstClock * clock, gpointer user_data)
+{
+  return GST_CLOCK_TIME_NONE;
 }
 
 static GstClockTime
@@ -188,7 +239,7 @@ gst_audio_clock_get_internal_time (GstClock * clock)
  * @clock: a #GstAudioClock
  *
  * Report the time as returned by the #GstAudioClockGetTimeFunc without applying
- * any offsets. 
+ * any offsets.
  *
  * Returns: the time as reported by the time function of the audio clock
  *
@@ -204,9 +255,14 @@ gst_audio_clock_get_time (GstClock * clock)
 
   result = aclock->func (clock, aclock->user_data);
   if (result == GST_CLOCK_TIME_NONE) {
+    GST_DEBUG_OBJECT (clock, "no time, reuse last");
     result = aclock->last_time - aclock->abidata.ABI.time_offset;
   }
 
+  GST_DEBUG_OBJECT (clock,
+      "result %" GST_TIME_FORMAT ", last_time %" GST_TIME_FORMAT,
+      GST_TIME_ARGS (result), GST_TIME_ARGS (aclock->last_time));
+
   return result;
 }
 
@@ -233,3 +289,26 @@ gst_audio_clock_adjust (GstClock * clock, GstClockTime time)
 
   return result;
 }
+
+/**
+ * gst_audio_clock_invalidate:
+ * @clock: a #GstAudioClock
+ *
+ * Invalidate the clock function. Call this function when the provided
+ * #GstAudioClockGetTimeFunc cannot be called anymore, for example, when the
+ * user_data becomes invalid.
+ *
+ * After calling this function, @clock will return the last returned time for
+ * the rest of its lifetime.
+ *
+ * Since: 0.10.31
+ */
+void
+gst_audio_clock_invalidate (GstClock * clock)
+{
+  GstAudioClock *aclock;
+
+  aclock = GST_AUDIO_CLOCK_CAST (clock);
+
+  aclock->func = gst_audio_clock_func_invalid;
+}
