@@ -44,11 +44,7 @@
  * SECTION:gstrtspurl
  * @short_description: handling RTSP urls
  *  
- * <refsect2>
- * <para>
  * Provides helper functions to handle RTSP urls.
- * </para>
- * </refsect2>
  *  
  * Last reviewed on 2007-07-25 (0.10.14)
  */
@@ -58,37 +54,31 @@
 
 #include "gstrtspurl.h"
 
-static void
-register_rtsp_url_type (GType * id)
-{
-  *id = g_boxed_type_register_static ("GstRTSPUrl",
-      (GBoxedCopyFunc) gst_rtsp_url_copy, (GBoxedFreeFunc) gst_rtsp_url_free);
-}
-
 GType
 gst_rtsp_url_get_type (void)
 {
-  static GType id;
-  static GOnce once = G_ONCE_INIT;
+  static volatile gsize url_type = 0;
 
-  g_once (&once, (GThreadFunc) register_rtsp_url_type, &id);
-  return id;
+  if (g_once_init_enter (&url_type)) {
+    GType tmp = g_boxed_type_register_static ("GstRTSPUrl",
+        (GBoxedCopyFunc) gst_rtsp_url_copy, (GBoxedFreeFunc) gst_rtsp_url_free);
+    g_once_init_leave (&url_type, tmp);
+  }
+
+  return (GType) url_type;
 }
 
-static const gchar *rtsp_url_schemes[] = {
-  "rtsp",
-  "rtspu",
-  "rtspt",
-  "rtsph",
-  NULL
-};
-
-static GstRTSPLowerTrans rtsp_url_transports[] = {
-  GST_RTSP_LOWER_TRANS_TCP | GST_RTSP_LOWER_TRANS_UDP |
-      GST_RTSP_LOWER_TRANS_UDP_MCAST,
-  GST_RTSP_LOWER_TRANS_UDP | GST_RTSP_LOWER_TRANS_UDP_MCAST,
-  GST_RTSP_LOWER_TRANS_TCP,
-  GST_RTSP_LOWER_TRANS_HTTP | GST_RTSP_LOWER_TRANS_TCP,
+static const struct
+{
+  const char scheme[6];
+  GstRTSPLowerTrans transports;
+} rtsp_schemes_map[] = {
+  {
+  "rtsp", GST_RTSP_LOWER_TRANS_TCP | GST_RTSP_LOWER_TRANS_UDP |
+        GST_RTSP_LOWER_TRANS_UDP_MCAST}, {
+  "rtspu", GST_RTSP_LOWER_TRANS_UDP | GST_RTSP_LOWER_TRANS_UDP_MCAST}, {
+  "rtspt", GST_RTSP_LOWER_TRANS_TCP}, {
+  "rtsph", GST_RTSP_LOWER_TRANS_HTTP | GST_RTSP_LOWER_TRANS_TCP}
 };
 
 /* format is rtsp[u]://[user:passwd@]host[:port]/abspath[?query] where host
@@ -113,7 +103,7 @@ gst_rtsp_url_parse (const gchar * urlstr, GstRTSPUrl ** url)
   GstRTSPUrl *res;
   gchar *p, *delim, *at, *col;
   gchar *host_end = NULL;
-  guint scheme;
+  guint i;
 
   g_return_val_if_fail (urlstr != NULL, GST_RTSP_EINVAL);
   g_return_val_if_fail (url != NULL, GST_RTSP_EINVAL);
@@ -126,9 +116,9 @@ gst_rtsp_url_parse (const gchar * urlstr, GstRTSPUrl ** url)
   if (col == NULL)
     goto invalid;
 
-  for (scheme = 0; rtsp_url_schemes[scheme] != NULL; scheme++) {
-    if (g_ascii_strncasecmp (rtsp_url_schemes[scheme], p, col - p) == 0) {
-      res->transports = rtsp_url_transports[scheme];
+  for (i = 0; i < G_N_ELEMENTS (rtsp_schemes_map); i++) {
+    if (g_ascii_strncasecmp (rtsp_schemes_map[i].scheme, p, col - p) == 0) {
+      res->transports = rtsp_schemes_map[i].transports;
       p = col + 3;
       break;
     }
