@@ -192,12 +192,15 @@ static const gchar *genres[] = {
   "Synthpop"
 };
 
-static GstTagEntryMatch tag_matches[] = {
+static const GstTagEntryMatch tag_matches[] = {
   {GST_TAG_TITLE, "TIT2"},
   {GST_TAG_ALBUM, "TALB"},
   {GST_TAG_TRACK_NUMBER, "TRCK"},
   {GST_TAG_ARTIST, "TPE1"},
+  {GST_TAG_ALBUM_ARTIST, "TPE2"},
+  {GST_TAG_COMPOSER, "TCOM"},
   {GST_TAG_COPYRIGHT, "TCOP"},
+  {GST_TAG_COPYRIGHT_URI, "WCOP"},
   {GST_TAG_GENRE, "TCON"},
   {GST_TAG_DATE, "TDRC"},
   {GST_TAG_COMMENT, "COMM"},
@@ -205,6 +208,11 @@ static GstTagEntryMatch tag_matches[] = {
   {GST_TAG_DURATION, "TLEN"},
   {GST_TAG_ISRC, "TSRC"},
   {GST_TAG_IMAGE, "APIC"},
+  {GST_TAG_ENCODER, "TSSE"},
+  {GST_TAG_BEATS_PER_MINUTE, "TBPM"},
+  {GST_TAG_ARTIST_SORTNAME, "TSOP"},
+  {GST_TAG_ALBUM_SORTNAME, "TSOA"},
+  {GST_TAG_TITLE_SORTNAME, "TSOT"},
   {NULL, NULL}
 };
 
@@ -236,24 +244,34 @@ gst_tag_from_id3_tag (const gchar * id3_tag)
   return NULL;
 }
 
-static GstTagEntryMatch user_tag_matches[] = {
+static const GstTagEntryMatch user_tag_matches[] = {
   /* musicbrainz identifiers being used in the real world (foobar2000) */
   {GST_TAG_MUSICBRAINZ_ARTISTID, "TXXX|musicbrainz_artistid"},
   {GST_TAG_MUSICBRAINZ_ALBUMID, "TXXX|musicbrainz_albumid"},
   {GST_TAG_MUSICBRAINZ_ALBUMARTISTID, "TXXX|musicbrainz_albumartistid"},
   {GST_TAG_MUSICBRAINZ_TRMID, "TXXX|musicbrainz_trmid"},
+  {GST_TAG_CDDA_MUSICBRAINZ_DISCID, "TXXX|musicbrainz_discid"},
   /* musicbrainz identifiers according to spec no one pays
    * attention to (http://musicbrainz.org/docs/specs/metadata_tags.html) */
   {GST_TAG_MUSICBRAINZ_ARTISTID, "TXXX|MusicBrainz Artist Id"},
   {GST_TAG_MUSICBRAINZ_ALBUMID, "TXXX|MusicBrainz Album Id"},
   {GST_TAG_MUSICBRAINZ_ALBUMARTISTID, "TXXX|MusicBrainz Album Artist Id"},
   {GST_TAG_MUSICBRAINZ_TRMID, "TXXX|MusicBrainz TRM Id"},
+  /* according to: http://wiki.musicbrainz.org/MusicBrainzTag (yes, no space
+   * before 'ID' and not 'Id' either this time, yay for consistency) */
+  {GST_TAG_CDDA_MUSICBRAINZ_DISCID, "TXXX|MusicBrainz DiscID"},
   /* foobar2000 uses these identifiers to store gain/peak information in
    * ID3v2 tags <= v2.3.0. In v2.4.0 there's the RVA2 frame for that */
   {GST_TAG_TRACK_GAIN, "TXXX|replaygain_track_gain"},
   {GST_TAG_TRACK_PEAK, "TXXX|replaygain_track_peak"},
   {GST_TAG_ALBUM_GAIN, "TXXX|replaygain_album_gain"},
-  {GST_TAG_ALBUM_PEAK, "TXXX|replaygain_album_peak"}
+  {GST_TAG_ALBUM_PEAK, "TXXX|replaygain_album_peak"},
+  /* the following two are more or less made up, there seems to be little
+   * evidence that any popular application is actually putting this info
+   * into TXXX frames; the first one comes from a musicbrainz wiki 'proposed
+   * tags' page, the second one is analogue to the vorbis/ape/flac tag. */
+  {GST_TAG_CDDA_CDDB_DISCID, "TXXX|discid"},
+  {GST_TAG_CDDA_CDDB_DISCID, "TXXX|CDDB DiscID"}
 };
 
 /**
@@ -276,7 +294,8 @@ gst_tag_from_id3_user_tag (const gchar * type, const gchar * id3_user_tag)
 
   for (i = 0; i < G_N_ELEMENTS (user_tag_matches); ++i) {
     if (strncmp (type, user_tag_matches[i].original_tag, 4) == 0 &&
-        strcmp (id3_user_tag, user_tag_matches[i].original_tag + 5) == 0) {
+        g_ascii_strcasecmp (id3_user_tag,
+            user_tag_matches[i].original_tag + 5) == 0) {
       GST_LOG ("Mapped ID3v2 user tag '%s' to GStreamer tag '%s'",
           user_tag_matches[i].original_tag, user_tag_matches[i].gstreamer_tag);
       return user_tag_matches[i].gstreamer_tag;
@@ -317,66 +336,14 @@ static void
 gst_tag_extract_id3v1_string (GstTagList * list, const gchar * tag,
     const gchar * start, const guint size)
 {
-  const gchar *env;
-  gsize bytes_read;
+  const gchar *env_vars[] = { "GST_ID3V1_TAG_ENCODING",
+    "GST_ID3_TAG_ENCODING", "GST_TAG_ENCODING", NULL
+  };
   gchar *utf8;
 
-  /* Should we try the charsets specified
-   * via environment variables FIRST ? */
-  if (g_utf8_validate (start, size, NULL)) {
-    utf8 = g_strndup (start, size);
-    goto beach;
-  }
+  utf8 = gst_tag_freeform_string_to_utf8 (start, size, env_vars);
 
-  env = g_getenv ("GST_ID3V1_TAG_ENCODING");
-  if (!env || *env == '\0')
-    env = g_getenv ("GST_ID3_TAG_ENCODING");
-  if (!env || *env == '\0')
-    env = g_getenv ("GST_TAG_ENCODING");
-
-  /* Try charsets specified via the environment */
-  if (env && *env != '\0') {
-    gchar **c, **csets;
-
-    csets = g_strsplit (env, G_SEARCHPATH_SEPARATOR_S, -1);
-
-    for (c = csets; c && *c; ++c) {
-      if ((utf8 =
-              g_convert (start, size, "UTF-8", *c, &bytes_read, NULL, NULL))) {
-        if (bytes_read == size) {
-          g_strfreev (csets);
-          goto beach;
-        }
-        g_free (utf8);
-        utf8 = NULL;
-      }
-    }
-  }
-  /* Try current locale (if not UTF-8) */
-  if (!g_get_charset (&env)) {
-    if ((utf8 = g_locale_to_utf8 (start, size, &bytes_read, NULL, NULL))) {
-      if (bytes_read == size) {
-        goto beach;
-      }
-      g_free (utf8);
-      utf8 = NULL;
-    }
-  }
-
-  /* Try ISO-8859-1 */
-  utf8 =
-      g_convert (start, size, "UTF-8", "ISO-8859-1", &bytes_read, NULL, NULL);
-  if (utf8 != NULL && bytes_read == size) {
-    goto beach;
-  }
-
-  g_free (utf8);
-  return;
-
-beach:
-
-  g_strchomp (utf8);
-  if (utf8 && utf8[0] != '\0') {
+  if (utf8 && *utf8 != '\0') {
     gst_tag_list_add (list, GST_TAG_MERGE_REPLACE, tag, utf8, NULL);
   }
 
@@ -416,7 +383,7 @@ gst_tag_list_new_from_id3v1 (const guint8 * data)
     gst_tag_list_add (list, GST_TAG_MERGE_REPLACE, GST_TAG_DATE, date, NULL);
     g_date_free (date);
   }
-  if (data[125] == 0) {
+  if (data[125] == 0 && data[126] != 0) {
     gst_tag_extract_id3v1_string (list, GST_TAG_COMMENT, (gchar *) & data[97],
         28);
     gst_tag_list_add (list, GST_TAG_MERGE_REPLACE, GST_TAG_TRACK_NUMBER,
@@ -425,7 +392,7 @@ gst_tag_list_new_from_id3v1 (const guint8 * data)
     gst_tag_extract_id3v1_string (list, GST_TAG_COMMENT, (gchar *) & data[97],
         30);
   }
-  if (data[127] < gst_tag_id3_genre_count ()) {
+  if (data[127] < gst_tag_id3_genre_count () && !gst_tag_list_is_empty (list)) {
     gst_tag_list_add (list, GST_TAG_MERGE_REPLACE, GST_TAG_GENRE,
         gst_tag_id3_genre_get (data[127]), NULL);
   }
@@ -462,3 +429,58 @@ gst_tag_id3_genre_get (const guint id)
     return NULL;
   return genres[id];
 }
+
+/**
+ * gst_tag_list_add_id3_image:
+ * @tag_list: a tag list
+ * @image_data: the (encoded) image
+ * @image_data_len: the length of the encoded image data at @image_data
+ * @id3_picture_type: picture type as per the ID3 (v2.4.0) specification for
+ *    the APIC frame (0 = unknown/other)
+ *
+ * Adds an image from an ID3 APIC frame (or similar, such as used in FLAC)
+ * to the given tag list. Also see gst_tag_image_data_to_image_buffer() for
+ * more information on image tags in GStreamer.
+ *
+ * Returns: %TRUE if the image was processed, otherwise %FALSE
+ *
+ * Since: 0.10.20
+ */
+gboolean
+gst_tag_list_add_id3_image (GstTagList * tag_list, const guint8 * image_data,
+    guint image_data_len, guint id3_picture_type)
+{
+  GstTagImageType tag_image_type;
+  const gchar *tag_name;
+  GstBuffer *image;
+
+  g_return_val_if_fail (GST_IS_TAG_LIST (tag_list), FALSE);
+  g_return_val_if_fail (image_data != NULL, FALSE);
+  g_return_val_if_fail (image_data_len > 0, FALSE);
+
+  if (id3_picture_type == 0x01 || id3_picture_type == 0x02) {
+    /* file icon for preview. Don't add image-type to caps, since there
+     * is only supposed to be one of these, and the type is already indicated
+     * via the special tag */
+    tag_name = GST_TAG_PREVIEW_IMAGE;
+    tag_image_type = GST_TAG_IMAGE_TYPE_NONE;
+  } else {
+    tag_name = GST_TAG_IMAGE;
+
+    /* Remap the ID3v2 APIC type our ImageType enum */
+    if (id3_picture_type >= 0x3 && id3_picture_type <= 0x14)
+      tag_image_type = (GstTagImageType) (id3_picture_type - 2);
+    else
+      tag_image_type = GST_TAG_IMAGE_TYPE_UNDEFINED;
+  }
+
+  image = gst_tag_image_data_to_image_buffer (image_data, image_data_len,
+      tag_image_type);
+
+  if (image == NULL)
+    return FALSE;
+
+  gst_tag_list_add (tag_list, GST_TAG_MERGE_APPEND, tag_name, image, NULL);
+  gst_buffer_unref (image);
+  return TRUE;
+}
