@@ -26,6 +26,13 @@
  * When using decodebin in your application, connect a signal handler to
  * #GstDecodeBin::new-decoded-pad and connect your sinks from within the
  * callback function.
+ *
+ * <note>
+ * This element is deprecated and no longer supported. You should use the
+ * #uridecodebin or #decodebin2 element instead (or, even better: #playbin2).
+ * </note>
+ *
+ * Deprecated: use uridecodebin or decodebin2 instead.
  */
 
 #ifdef HAVE_CONFIG_H
@@ -187,12 +194,6 @@ static gboolean is_demuxer_element (GstElement * srcelement);
 static GstElementClass *parent_class;
 static guint gst_decode_bin_signals[LAST_SIGNAL] = { 0 };
 
-static const GstElementDetails gst_decode_bin_details =
-GST_ELEMENT_DETAILS ("Decoder Bin",
-    "Generic/Bin/Decoder",
-    "Autoplug and decode to raw media",
-    "Wim Taymans <wim.taymans@gmail.com>");
-
 
 static GType
 gst_decode_bin_get_type (void)
@@ -290,7 +291,10 @@ gst_decode_bin_class_init (GstDecodeBinClass * klass)
   gst_element_class_add_pad_template (gstelement_klass,
       gst_static_pad_template_get (&decoder_bin_src_template));
 
-  gst_element_class_set_details (gstelement_klass, &gst_decode_bin_details);
+  gst_element_class_set_details_simple (gstelement_klass,
+      "Decoder Bin", "Generic/Bin/Decoder",
+      "Autoplug and decode to raw media",
+      "Wim Taymans <wim.taymans@gmail.com>");
 
   gstelement_klass->change_state =
       GST_DEBUG_FUNCPTR (gst_decode_bin_change_state);
@@ -400,7 +404,8 @@ gst_decode_bin_init (GstDecodeBin * decode_bin)
     pad = gst_element_get_static_pad (decode_bin->typefind, "sink");
 
     /* ghost the sink pad to ourself */
-    gpad = gst_ghost_pad_new ("sink", pad);
+    gpad = gst_ghost_pad_new_from_template ("sink", pad,
+        gst_static_pad_template_get (&decoder_bin_sink_template));
     gst_pad_set_active (gpad, TRUE);
     gst_element_add_pad (GST_ELEMENT (decode_bin), gpad);
 
@@ -634,23 +639,21 @@ find_compatibles (GstDecodeBin * decode_bin, const GstCaps * caps)
 
       /* we only care about the sink templates */
       if (templ->direction == GST_PAD_SINK) {
-        GstCaps *intersect;
+        gboolean can_intersect;
         GstCaps *tmpl_caps;
 
         /* try to intersect the caps with the caps of the template */
         tmpl_caps = gst_static_caps_get (&templ->static_caps);
 
-        intersect = gst_caps_intersect (caps, tmpl_caps);
+        can_intersect = gst_caps_can_intersect (caps, tmpl_caps);
         gst_caps_unref (tmpl_caps);
 
         /* check if the intersection is empty */
-        if (!gst_caps_is_empty (intersect)) {
+        if (can_intersect) {
           /* non empty intersection, we can use this element */
           to_try = g_list_prepend (to_try, factory);
-          gst_caps_unref (intersect);
           break;
         }
-        gst_caps_unref (intersect);
       }
     }
   }
@@ -912,7 +915,8 @@ close_pad_link (GstElement * element, GstPad * pad, GstCaps * caps,
     decode_bin->numpads++;
 
     /* make it a ghostpad */
-    ghost = gst_ghost_pad_new (padname, pad);
+    ghost = gst_ghost_pad_new_from_template (padname, pad,
+        gst_static_pad_template_get (&decoder_bin_src_template));
     gst_pad_set_active (ghost, TRUE);
     gst_element_add_pad (GST_ELEMENT (decode_bin), ghost);
 
