@@ -41,14 +41,6 @@
 GST_DEBUG_CATEGORY_STATIC (input_selector_debug);
 #define GST_CAT_DEFAULT input_selector_debug
 
-static const GstElementDetails gst_input_selector_details =
-GST_ELEMENT_DETAILS ("Input selector",
-    "Generic",
-    "N-to-1 input stream selectoring",
-    "Julien Moutte <julien@moutte.net>\n"
-    "Jan Schmidt <thaytan@mad.scientist.com>\n"
-    "Wim Taymans <wim.taymans@gmail.com>");
-
 static GstStaticPadTemplate gst_input_selector_sink_factory =
 GST_STATIC_PAD_TEMPLATE ("sink%d",
     GST_PAD_SINK,
@@ -154,7 +146,7 @@ static GstFlowReturn gst_selector_pad_chain (GstPad * pad, GstBuffer * buf);
 static GstFlowReturn gst_selector_pad_bufferalloc (GstPad * pad,
     guint64 offset, guint size, GstCaps * caps, GstBuffer ** buf);
 
-GType
+static GType
 gst_selector_pad_get_type (void)
 {
   static GType selector_pad_type = 0;
@@ -195,18 +187,20 @@ gst_selector_pad_class_init (GstSelectorPadClass * klass)
 
   g_object_class_install_property (gobject_class, PROP_PAD_RUNNING_TIME,
       g_param_spec_int64 ("running-time", "Running time",
-          "Running time of stream on pad", 0, G_MAXINT64, 0, G_PARAM_READABLE));
+          "Running time of stream on pad", 0, G_MAXINT64, 0,
+          G_PARAM_READABLE | G_PARAM_STATIC_STRINGS));
   g_object_class_install_property (gobject_class, PROP_PAD_TAGS,
       g_param_spec_boxed ("tags", "Tags",
           "The currently active tags on the pad", GST_TYPE_TAG_LIST,
-          G_PARAM_READABLE));
+          G_PARAM_READABLE | G_PARAM_STATIC_STRINGS));
   g_object_class_install_property (gobject_class, PROP_PAD_ACTIVE,
       g_param_spec_boolean ("active", "Active",
-          "If the pad is currently active", FALSE, G_PARAM_READABLE));
+          "If the pad is currently active", FALSE,
+          G_PARAM_READABLE | G_PARAM_STATIC_STRINGS));
   g_object_class_install_property (gobject_class, PROP_PAD_ALWAYS_OK,
       g_param_spec_boolean ("always-ok", "Always OK",
           "Make an inactive pad return OK instead of NOT_LINKED",
-          DEFAULT_PAD_ALWAYS_OK, G_PARAM_READWRITE));
+          DEFAULT_PAD_ALWAYS_OK, G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
 }
 
 static void
@@ -746,7 +740,12 @@ gst_input_selector_base_init (GstInputSelectorClass * klass)
 {
   GstElementClass *element_class = GST_ELEMENT_CLASS (klass);
 
-  gst_element_class_set_details (element_class, &gst_input_selector_details);
+  gst_element_class_set_details_simple (element_class,
+      "Input selector", "Generic",
+      "N-to-1 input stream selectoring",
+      "Julien Moutte <julien@moutte.net>, "
+      "Jan Schmidt <thaytan@mad.scientist.com>, "
+      "Wim Taymans <wim.taymans@gmail.com>");
   gst_element_class_add_pad_template (element_class,
       gst_static_pad_template_get (&gst_input_selector_sink_factory));
   gst_element_class_add_pad_template (element_class,
@@ -761,6 +760,9 @@ gst_input_selector_class_init (GstInputSelectorClass * klass)
 
   parent_class = g_type_class_peek_parent (klass);
 
+  /* FIXME: remove after confirming it is safe now */
+  g_type_class_ref (gst_selector_pad_get_type ());
+
   gobject_class->dispose = gst_input_selector_dispose;
 
   gobject_class->set_property = gst_input_selector_set_property;
