@@ -124,14 +124,6 @@
 
 #define NOT_IMPLEMENTED 0
 
-/* elementfactory information */
-static const GstElementDetails gst_multi_fd_sink_details =
-GST_ELEMENT_DETAILS ("Multi filedescriptor sink",
-    "Sink/Network",
-    "Send data to multiple filedescriptors",
-    "Thomas Vander Stichele <thomas at apestaart dot org>, "
-    "Wim Taymans <wim@fluendo.com>");
-
 static GstStaticPadTemplate sinktemplate = GST_STATIC_PAD_TEMPLATE ("sink",
     GST_PAD_SINK,
     GST_PAD_ALWAYS,
@@ -218,6 +210,8 @@ enum
 
   PROP_RESEND_STREAMHEADER,
 
+  PROP_NUM_FDS,
+
   PROP_LAST
 };
 
@@ -364,7 +358,11 @@ gst_multi_fd_sink_base_init (gpointer g_class)
   gst_element_class_add_pad_template (element_class,
       gst_static_pad_template_get (&sinktemplate));
 
-  gst_element_class_set_details (element_class, &gst_multi_fd_sink_details);
+  gst_element_class_set_details_simple (element_class,
+      "Multi filedescriptor sink", "Sink/Network",
+      "Send data to multiple filedescriptors",
+      "Thomas Vander Stichele <thomas at apestaart dot org>, "
+      "Wim Taymans <wim@fluendo.com>");
 }
 
 static void
@@ -383,7 +381,8 @@ gst_multi_fd_sink_class_init (GstMultiFdSinkClass * klass)
   gobject_class->finalize = gst_multi_fd_sink_finalize;
 
   g_object_class_install_property (gobject_class, PROP_PROTOCOL,
-      g_param_spec_enum ("protocol", "Protocol", "The protocol to wrap data in",
+      g_param_spec_enum ("protocol", "Protocol", "The protocol to wrap data in"
+          ". GDP protocol here is deprecated. Please use gdppay element.",
           GST_TYPE_TCP_PROTOCOL, DEFAULT_PROTOCOL,
           G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
 
@@ -520,6 +519,11 @@ gst_multi_fd_sink_class_init (GstMultiFdSinkClass * klass)
           DEFAULT_RESEND_STREAMHEADER,
           G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
 
+  g_object_class_install_property (gobject_class, PROP_NUM_FDS,
+      g_param_spec_uint ("num-fds", "Number of fds",
+          "The current number of client file descriptors.",
+          0, G_MAXUINT, 0, G_PARAM_READABLE | G_PARAM_STATIC_STRINGS));
+
   /**
    * GstMultiFdSink::add:
    * @gstmultifdsink: the multifdsink element to emit this signal on
@@ -2798,6 +2802,9 @@ gst_multi_fd_sink_get_property (GObject * object, guint prop_id, GValue * value,
     case PROP_RESEND_STREAMHEADER:
       g_value_set_boolean (value, multifdsink->resend_streamheader);
       break;
+    case PROP_NUM_FDS:
+      g_value_set_uint (value, g_hash_table_size (multifdsink->fd_hash));
+      break;
 
     default:
       G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
